@@ -32,7 +32,6 @@
 
 #define CONNMAN_API_SUBJECT_TO_CHANGE
 #include <connman/plugin.h>
-#include <connman/element.h>
 #include <connman/device.h>
 #include <connman/network.h>
 #include <connman/ipconfig.h>
@@ -90,13 +89,17 @@ struct modem_data {
 	connman_bool_t registered;
 	connman_bool_t roaming;
 	uint8_t strength, has_strength;
+	char *operator;
 };
 
 struct network_info {
 	struct connman_network *network;
 
-	enum connman_ipconfig_method method;
-	struct connman_ipaddress ipaddress;
+	enum connman_ipconfig_method ipv4_method;
+	struct connman_ipaddress ipv4_address;
+
+	enum connman_ipconfig_method ipv6_method;
+	struct connman_ipaddress ipv6_address;
 };
 
 static int modem_probe(struct connman_device *device)
@@ -250,6 +253,36 @@ static void update_modem_online(struct modem_data *modem,
 		connman_device_set_powered(modem->device, online);
 }
 
+static void remove_device_networks(struct connman_device *device)
+{
+	GHashTableIter iter;
+	gpointer key, value;
+	GSList *info_list = NULL;
+	GSList *list;
+
+	if (network_hash == NULL)
+		return;
+
+	g_hash_table_iter_init(&iter, network_hash);
+
+	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
+		struct network_info *info = value;
+
+		if (connman_network_get_device(info->network) != device)
+			continue;
+
+		info_list = g_slist_append(info_list, info);
+	}
+
+	for (list = info_list; list != NULL; list = list->next) {
+		struct network_info *info = list->data;
+
+		connman_device_remove_network(device, info->network);
+	}
+
+	g_slist_free(info_list);
+}
+
 static void set_online_reply(DBusPendingCall *call, void *user_data)
 {
 	struct modem_data *modem;
@@ -282,6 +315,9 @@ static void set_online_reply(DBusPendingCall *call, void *user_data)
 	if (modem->pending_online)
 		update_modem_online(modem, result);
 
+	if (result == FALSE)
+		remove_device_networks(modem->device);
+
 	dbus_message_unref(reply);
 
 	dbus_pending_call_unref(call);
@@ -339,7 +375,8 @@ static void modem_remove_device(struct modem_data *modem)
 	if (modem->device == NULL)
 		return;
 
-	connman_device_remove_all_networks(modem->device);
+	remove_device_networks(modem->device);
+
 	connman_device_unregister(modem->device);
 	connman_device_unref(modem->device);
 
@@ -353,6 +390,7 @@ static void remove_modem(gpointer data)
 	modem_remove_device(modem);
 
 	g_free(modem->path);
+	g_free(modem->operator);
 
 	g_free(modem);
 }
@@ -360,6 +398,11 @@ static void remove_modem(gpointer data)
 static void remove_network(gpointer data)
 {
 	struct network_info *info = data;
+	struct connman_device *device;
+
+	device = connman_network_get_device(info->network);
+	if (device != NULL)
+		connman_device_remove_network(device, info->network);
 
 	connman_network_unref(info->network);
 
@@ -413,28 +456,51 @@ static gboolean pending_network_is_available(struct connman_network *network)
 static void set_connected(struct network_info *info,
 				connman_bool_t connected)
 {
+	gboolean setip = FALSE;
+
 	DBG("network %p connected %d", info->network, connected);
 
-	switch (info->method) {
+	switch (info->ipv4_method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 	case CONNMAN_IPCONFIG_METHOD_OFF:
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 	case CONNMAN_IPCONFIG_METHOD_AUTO:
-		return;
+		break;
 
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
-		connman_network_set_ipv4_method(info->network, info->method);
-		connman_network_set_ipaddress(info->network, &info->ipaddress);
-
+		connman_network_set_ipv4_method(info->network,
+							info->ipv4_method);
+		connman_network_set_ipaddress(info->network,
+							&info->ipv4_address);
+		setip = TRUE;
 		break;
 
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
-		connman_network_set_ipv4_method(info->network, info->method);
-
+		connman_network_set_ipv4_method(info->network,
+							info->ipv4_method);
+		setip = TRUE;
 		break;
 	}
 
-	connman_network_set_connected(info->network, connected);
+	switch (info->ipv6_method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
+		break;
+
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+		connman_network_set_ipv6_method(info->network,
+							info->ipv6_method);
+		connman_network_set_ipaddress(info->network,
+							&info->ipv6_address);
+		setip = TRUE;
+		break;
+	}
+
+	if (setip == TRUE)
+		connman_network_set_connected(info->network, connected);
 }
 
 static void set_active_reply(DBusPendingCall *call, void *user_data)
@@ -446,10 +512,13 @@ static void set_active_reply(DBusPendingCall *call, void *user_data)
 
 	info = g_hash_table_lookup(network_hash, path);
 
-	DBG("path %s network %p", path, info->network);
-
 	reply = dbus_pending_call_steal_reply(call);
 
+	if (info == NULL)
+		goto done;
+
+	DBG("path %s network %p", path, info->network);
+
 	if (!pending_network_is_available(info->network))
 		goto done;
 
@@ -459,13 +528,11 @@ static void set_active_reply(DBusPendingCall *call, void *user_data)
 		connman_error("SetProperty(Active) %s %s",
 				error.name, error.message);
 
-		if (connman_network_get_index(info->network) < 0)
-			connman_network_set_error(info->network,
+		connman_network_set_error(info->network,
 				CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
 
 		dbus_error_free(&error);
-	} else if (connman_network_get_index(info->network) >= 0)
-		set_connected(info, TRUE);
+	}
 
 done:
 	dbus_message_unref(reply);
@@ -560,8 +627,209 @@ static struct connman_network_driver network_driver = {
 	.disconnect	= network_disconnect,
 };
 
-static void update_settings(DBusMessageIter *array,
-				struct network_info *info);
+static void get_dns(DBusMessageIter *array, struct network_info *info)
+{
+	DBusMessageIter entry;
+	gchar *nameservers = NULL, *nameservers_old = NULL;
+
+	DBG("");
+
+
+	dbus_message_iter_recurse(array, &entry);
+
+	while (dbus_message_iter_get_arg_type(&entry) == DBUS_TYPE_STRING) {
+		const char *dns;
+
+		dbus_message_iter_get_basic(&entry, &dns);
+
+		DBG("dns %s", dns);
+
+		if (nameservers == NULL) {
+
+			nameservers = g_strdup(dns);
+		} else {
+
+			nameservers_old = nameservers;
+			nameservers = g_strdup_printf("%s %s",
+						nameservers_old, dns);
+			g_free(nameservers_old);
+		}
+
+		dbus_message_iter_next(&entry);
+	}
+
+	connman_network_set_nameservers(info->network, nameservers);
+
+	g_free(nameservers);
+}
+
+static void update_ipv4_settings(DBusMessageIter *array,
+				struct network_info *info)
+{
+	DBusMessageIter dict;
+	char *address = NULL, *netmask = NULL, *gateway = NULL;
+	const char *interface = NULL;
+
+	DBG("network %p", info->network);
+
+	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
+		return;
+
+	dbus_message_iter_recurse(array, &dict);
+
+	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
+		DBusMessageIter entry, value;
+		const char *key, *val;
+
+		dbus_message_iter_recurse(&dict, &entry);
+		dbus_message_iter_get_basic(&entry, &key);
+
+		DBG("key %s", key);
+
+		dbus_message_iter_next(&entry);
+		dbus_message_iter_recurse(&entry, &value);
+
+		if (g_str_equal(key, "Interface") == TRUE) {
+			int index;
+
+			dbus_message_iter_get_basic(&value, &interface);
+
+			DBG("interface %s", interface);
+
+			index = connman_inet_ifindex(interface);
+			if (index >= 0) {
+				connman_network_set_index(info->network, index);
+			} else {
+				connman_error("Can not find interface %s",
+								interface);
+				break;
+			}
+		} else if (g_str_equal(key, "Method") == TRUE) {
+			const char *method;
+
+			dbus_message_iter_get_basic(&value, &method);
+
+			if (g_strcmp0(method, "static") == 0) {
+
+				info->ipv4_method =
+					CONNMAN_IPCONFIG_METHOD_FIXED;
+			} else if (g_strcmp0(method, "dhcp") == 0) {
+
+				info->ipv4_method =
+					CONNMAN_IPCONFIG_METHOD_DHCP;
+				break;
+			}
+		} else if (g_str_equal(key, "Address") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			address = g_strdup(val);
+
+			DBG("address %s", address);
+		} else if (g_str_equal(key, "Netmask") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			netmask = g_strdup(val);
+
+			DBG("netmask %s", netmask);
+		} else if (g_str_equal(key, "DomainNameServers") == TRUE) {
+
+			get_dns(&value, info);
+		} else if (g_str_equal(key, "Gateway") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			gateway = g_strdup(val);
+
+			DBG("gateway %s", gateway);
+		}
+
+		dbus_message_iter_next(&dict);
+	}
+
+
+	if (info->ipv4_method == CONNMAN_IPCONFIG_METHOD_FIXED) {
+		connman_ipaddress_set_ipv4(&info->ipv4_address, address,
+						netmask, gateway);
+	}
+
+	g_free(address);
+	g_free(netmask);
+	g_free(gateway);
+}
+
+static void update_ipv6_settings(DBusMessageIter *array,
+				struct network_info *info)
+{
+	DBusMessageIter dict;
+	char *address = NULL, *gateway = NULL;
+	unsigned char prefix_length;
+	const char *interface = NULL;
+
+	DBG("network %p", info->network);
+
+	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
+		return;
+
+	dbus_message_iter_recurse(array, &dict);
+
+	info->ipv6_method = CONNMAN_IPCONFIG_METHOD_FIXED;
+
+	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
+		DBusMessageIter entry, value;
+		const char *key, *val;
+
+		dbus_message_iter_recurse(&dict, &entry);
+		dbus_message_iter_get_basic(&entry, &key);
+
+		DBG("key %s", key);
+
+		dbus_message_iter_next(&entry);
+		dbus_message_iter_recurse(&entry, &value);
+
+		if (g_str_equal(key, "Interface") == TRUE) {
+			int index;
+
+			dbus_message_iter_get_basic(&value, &interface);
+
+			DBG("interface %s", interface);
+
+			index = connman_inet_ifindex(interface);
+			if (index >= 0) {
+				connman_network_set_index(info->network, index);
+			} else {
+				connman_error("Can not find interface %s",
+								interface);
+				break;
+			}
+		} else if (g_str_equal(key, "Address") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			address = g_strdup(val);
+
+			DBG("address %s", address);
+		} else if (g_str_equal(key, "PrefixLength") == TRUE) {
+			dbus_message_iter_get_basic(&value, &prefix_length);
+
+			DBG("prefix length %d", prefix_length);
+		} else if (g_str_equal(key, "DomainNameServers") == TRUE) {
+
+			get_dns(&value, info);
+		} else if (g_str_equal(key, "Gateway") == TRUE) {
+			dbus_message_iter_get_basic(&value, &val);
+
+			gateway = g_strdup(val);
+
+			DBG("gateway %s", gateway);
+		}
+
+		dbus_message_iter_next(&dict);
+	}
+
+	connman_ipaddress_set_ipv6(&info->ipv6_address, address,
+						prefix_length, gateway);
+
+	g_free(address);
+	g_free(gateway);
+}
 
 static int add_network(struct connman_device *device,
 			const char *path, DBusMessageIter *dict)
@@ -570,8 +838,6 @@ static int add_network(struct connman_device *device,
 	struct connman_network *network;
 	struct network_info *info;
 	char *ident;
-	const char *hash_path;
-	char const *operator;
 	dbus_bool_t active = FALSE;
 
 	DBG("modem %p device %p path %s", modem, device, path);
@@ -601,28 +867,23 @@ static int add_network(struct connman_device *device,
 		return -ENOMEM;
 	}
 
-	connman_ipaddress_clear(&info->ipaddress);
+	connman_ipaddress_clear(&info->ipv4_address);
+	connman_ipaddress_clear(&info->ipv6_address);
 	info->network = network;
 
 	connman_network_set_string(network, "Path", path);
-	hash_path = connman_network_get_string(network, "Path");
-	if (hash_path == NULL) {
-		connman_network_unref(network);
-		g_free(info);
-		return -EIO;
-	}
 
 	create_service(network);
 
-	connman_network_ref(network);
-	g_hash_table_insert(network_hash, (char *) hash_path, info);
+	g_hash_table_insert(network_hash, g_strdup(path), info);
 
 	connman_network_set_available(network, TRUE);
 	connman_network_set_index(network, -1);
 
-	operator = connman_device_get_string(device, "Operator");
-	if (operator)
-		connman_network_set_name(network, operator);
+	if (modem->operator)
+		connman_network_set_name(network, modem->operator);
+	else
+		connman_network_set_name(network, "");
 
 	if (modem->has_strength)
 		connman_network_set_strength(network, modem->strength);
@@ -645,29 +906,29 @@ static int add_network(struct connman_device *device,
 			dbus_message_iter_get_basic(&value, &type);
 			if (g_strcmp0(type, "internet") != 0) {
 				DBG("path %p type %s", path, type);
-				goto error;
+				g_hash_table_remove(network_hash, path);
+				return -EIO;
 			}
 		} else if (g_str_equal(key, "Settings"))
-			update_settings(&value, info);
+			update_ipv4_settings(&value, info);
+		else if (g_str_equal(key, "IPv6.Settings"))
+			update_ipv6_settings(&value, info);
 		else if (g_str_equal(key, "Active") == TRUE)
 			dbus_message_iter_get_basic(&value, &active);
 
 		dbus_message_iter_next(dict);
 	}
 
-	if (connman_device_add_network(device, network) != 0)
-		goto error;
+	if (connman_device_add_network(device, network) != 0) {
+		g_hash_table_remove(network_hash, path);
+		return -EIO;
+	}
 
 	/* Connect only if requested to do so */
 	if (active && connman_network_get_connecting(network) == TRUE)
 		set_connected(info, active);
 
 	return 0;
-
-error:
-	connman_network_unref(network);
-	g_hash_table_remove(network_hash, hash_path);
-	return -EIO;
 }
 
 static void check_networks_reply(DBusPendingCall *call, void *user_data)
@@ -754,7 +1015,9 @@ static void modem_operator_name_changed(struct modem_data *modem,
 	if (device == NULL)
 		return;
 
-	connman_device_set_string(device, "Operator", name);
+	if (modem->operator != NULL)
+		g_free(modem->operator);
+	modem->operator = g_strdup(name);
 
 	for (g_hash_table_iter_init(&i, network_hash);
 	     g_hash_table_iter_next(&i, NULL, &value);) {
@@ -974,10 +1237,15 @@ static void modem_gprs_changed(struct modem_data *modem,
 	if (g_str_equal(key, "Attached") == TRUE) {
 		DBG("Attached %d", value);
 
+		if (modem->attached == value)
+			return;
+
 		modem->attached = value;
 
-		if (value)
+		if (value == TRUE) {
 			modem_clear_network_errors(modem);
+			check_networks(modem);
+		}
 	} else if (g_str_equal(key, "Powered") == TRUE) {
 		DBG("Powered %d", value);
 
@@ -1022,7 +1290,6 @@ done:
 
 	dbus_pending_call_unref(call);
 
-	check_networks(modem);
 }
 
 static void check_gprs(struct modem_data *modem)
@@ -1199,7 +1466,6 @@ static void add_modem(const char *path, DBusMessageIter *prop)
 	struct modem_data *modem;
 	dbus_bool_t powered = FALSE;
 	dbus_bool_t online = FALSE;
-	dbus_bool_t has_online = FALSE;
 	dbus_bool_t locked = FALSE;
 	gboolean has_sim = FALSE;
 	gboolean has_reg = FALSE;
@@ -1234,10 +1500,7 @@ static void add_modem(const char *path, DBusMessageIter *prop)
 			dbus_message_iter_get_basic(&value, &powered);
 		else if (g_str_equal(key, "Lockdown") == TRUE)
 			dbus_message_iter_get_basic(&value, &locked);
-		else if (g_str_equal(key, "Online") == TRUE) {
-			has_online = TRUE;
-			dbus_message_iter_get_basic(&value, &online);
-		} else if (g_str_equal(key, "Interfaces") == TRUE) {
+		else if (g_str_equal(key, "Interfaces") == TRUE) {
 			has_sim = modem_has_sim(&value);
 			has_reg = modem_has_reg(&value);
 			has_gprs = modem_has_gprs(&value);
@@ -1319,7 +1582,7 @@ static void ofono_connect(DBusConnection *connection, void *user_data)
 						g_free, remove_modem);
 
 	network_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
-						NULL, remove_network);
+						g_free, remove_network);
 
 	call_ofono("/", OFONO_MANAGER_INTERFACE, GET_MODEMS,
 			manager_modems_reply, NULL, NULL,
@@ -1330,12 +1593,15 @@ static void ofono_disconnect(DBusConnection *connection, void *user_data)
 {
 	DBG("connection %p", connection);
 
-	if (modem_hash == NULL)
-		return;
+	if (modem_hash != NULL) {
+		g_hash_table_destroy(modem_hash);
+		modem_hash = NULL;
+	}
 
-	g_hash_table_destroy(modem_hash);
-
-	modem_hash = NULL;
+	if (network_hash != NULL) {
+		g_hash_table_destroy(network_hash);
+		network_hash = NULL;
+	}
 }
 
 static gboolean modem_changed(DBusConnection *connection, DBusMessage *message,
@@ -1497,6 +1763,10 @@ static gboolean context_added(DBusConnection *connection,
 	if (modem == NULL || modem->device == NULL)
 		return TRUE;
 
+	/* Create the related network only when we are attached */
+	if (modem->attached == FALSE)
+		return TRUE;
+
 	if (dbus_message_iter_init(message, &iter) == FALSE)
 		return TRUE;
 
@@ -1514,9 +1784,8 @@ static gboolean context_removed(DBusConnection *connection,
 				DBusMessage *message, void *user_data)
 {
 	const char *path = dbus_message_get_path(message);
-	const char *network_path, *identifier;
+	const char *network_path;
 	struct modem_data *modem;
-	struct network_info *info;
 	DBusMessageIter iter;
 
 	DBG("path %s", path);
@@ -1530,13 +1799,7 @@ static gboolean context_removed(DBusConnection *connection,
 
 	dbus_message_iter_get_basic(&iter, &network_path);
 
-	info = g_hash_table_lookup(network_hash, network_path);
-	if (info == NULL)
-		return TRUE;
-
-	identifier = connman_network_get_identifier(info->network);
-	connman_device_remove_network(modem->device, identifier);
-
+	g_hash_table_remove(network_hash, network_path);
 	return TRUE;
 }
 
@@ -1579,138 +1842,6 @@ static gboolean modem_removed(DBusConnection *connection,
 	return TRUE;
 }
 
-
-static void get_dns(DBusMessageIter *array, struct network_info *info)
-{
-	DBusMessageIter entry;
-	gchar *nameservers = NULL, *nameservers_old = NULL;
-
-	DBG("");
-
-
-	dbus_message_iter_recurse(array, &entry);
-
-	while (dbus_message_iter_get_arg_type(&entry) == DBUS_TYPE_STRING) {
-		const char *dns;
-
-		dbus_message_iter_get_basic(&entry, &dns);
-
-		DBG("dns %s", dns);
-
-		if (nameservers == NULL) {
-
-			nameservers = g_strdup(dns);
-		} else {
-
-			nameservers_old = nameservers;
-			nameservers = g_strdup_printf("%s %s",
-						nameservers_old, dns);
-			g_free(nameservers_old);
-		}
-
-		dbus_message_iter_next(&entry);
-	}
-
-	connman_network_set_nameservers(info->network, nameservers);
-
-	g_free(nameservers);
-}
-
-static void update_settings(DBusMessageIter *array,
-				struct network_info *info)
-{
-	DBusMessageIter dict;
-	char *address = NULL, *netmask = NULL, *gateway = NULL;
-	const char *interface = NULL;
-
-	DBG("network %p", info->network);
-
-	if (dbus_message_iter_get_arg_type(array) != DBUS_TYPE_ARRAY)
-		return;
-
-	dbus_message_iter_recurse(array, &dict);
-
-	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
-		DBusMessageIter entry, value;
-		const char *key, *val;
-
-		dbus_message_iter_recurse(&dict, &entry);
-		dbus_message_iter_get_basic(&entry, &key);
-
-		DBG("key %s", key);
-
-		dbus_message_iter_next(&entry);
-		dbus_message_iter_recurse(&entry, &value);
-
-		if (g_str_equal(key, "Interface") == TRUE) {
-			int index;
-
-			dbus_message_iter_get_basic(&value, &interface);
-
-			DBG("interface %s", interface);
-
-			index = connman_inet_ifindex(interface);
-			if (index >= 0) {
-				connman_network_set_index(info->network, index);
-			} else {
-				connman_error("Can not find interface %s",
-								interface);
-				break;
-			}
-		} else if (g_str_equal(key, "Method") == TRUE) {
-			const char *method;
-
-			dbus_message_iter_get_basic(&value, &method);
-
-			if (g_strcmp0(method, "static") == 0) {
-
-				info->method = CONNMAN_IPCONFIG_METHOD_FIXED;
-			} else if (g_strcmp0(method, "dhcp") == 0) {
-
-				info->method = CONNMAN_IPCONFIG_METHOD_DHCP;
-				break;
-			}
-		} else if (g_str_equal(key, "Address") == TRUE) {
-			dbus_message_iter_get_basic(&value, &val);
-
-			address = g_strdup(val);
-
-			DBG("address %s", address);
-		} else if (g_str_equal(key, "Netmask") == TRUE) {
-			dbus_message_iter_get_basic(&value, &val);
-
-			netmask = g_strdup(val);
-
-			DBG("netmask %s", netmask);
-		} else if (g_str_equal(key, "DomainNameServers") == TRUE) {
-
-			get_dns(&value, info);
-		} else if (g_str_equal(key, "Gateway") == TRUE) {
-			dbus_message_iter_get_basic(&value, &val);
-
-			gateway = g_strdup(val);
-
-			DBG("gateway %s", gateway);
-		}
-
-		dbus_message_iter_next(&dict);
-	}
-
-
-	if (info->method == CONNMAN_IPCONFIG_METHOD_FIXED) {
-		connman_ipaddress_set_ipv4(&info->ipaddress, address,
-						netmask, gateway);
-	}
-
-	/* deactive, oFono send NULL inteface before deactive signal */
-	if (interface == NULL)
-		connman_network_set_index(info->network, -1);
-
-	g_free(address);
-	g_free(netmask);
-	g_free(gateway);
-}
-
 static gboolean context_changed(DBusConnection *connection,
 					DBusMessage *message, void *user_data)
 {
@@ -1741,7 +1872,9 @@ static gboolean context_changed(DBusConnection *connection,
 	DBG("key %s", key);
 
 	if (g_str_equal(key, "Settings") == TRUE)
-		update_settings(&value, info);
+		update_ipv4_settings(&value, info);
+	else if (g_str_equal(key, "IPv6.Settings"))
+			update_ipv6_settings(&value, info);
 	else if (g_str_equal(key, "Active") == TRUE) {
 		dbus_bool_t active;
 
@@ -1749,10 +1882,8 @@ static gboolean context_changed(DBusConnection *connection,
 
 		if (active == FALSE)
 			set_connected(info, active);
-
-		/* Connect only if requested to do so */
-		if (active == TRUE &&
-			connman_network_get_connecting(info->network) == TRUE)
+		else if (connman_network_get_connecting(info->network) == TRUE)
+			/* Connect only if requested to do so */
 			set_connected(info, active);
 	}
 
